<?php
/**
 * MapEdit Viewer Code
 *
 * @project     MapEdit
 * @revision    $Id: preview.php,v 1.53.2.2 2004/07/21 13:23:46 zak Exp $
 * @purpose     This file contains functions and properties necessary for all
 *              gmap calculations and settings.
 * @author      William A. Bronsema, C.E.T. (bronsema@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

/*****************************************************************************
 * $Log: preview.php,v $
 * Revision 1.53.2.2  2004/07/21 13:23:46  zak
 * fixed some warnings
 *
 * Revision 1.53.2.1  2003/10/15 20:04:57  zak
 * updates for popup fixes
 * fixed code causing php notices
 *
 * Revision 1.53  2003/03/17 14:45:32  pspencer
 * fixed preview legend template problem when no template was set in the mapfile.
 *
 * Revision 1.52  2003/03/14 22:28:13  pspencer
 * fixed potential problem with temporary directory, image type and a notice
 * that was coming from checking for INPUT_COORD.
 *
 * Revision 1.51  2003/03/11 17:15:32  sacha
 * use current dir for template
 *
 * Revision 1.50  2003/02/28 20:51:57  pspencer
 * fixed layer status change problem
 *
 * Revision 1.49  2003/01/30 14:18:40  pspencer
 * use uppercase version of image type to prevent problems if the user puts the image type in lower case in the configuration file.
 *
 * Revision 1.48  2002/11/29 19:27:17  assefa
 * Use legend template from the map if it exists.
 *
 * Revision 1.47  2002/11/16 18:52:57  pspencer
 * changed default image type to PNG if image type isn't registered.
 *
 * Revision 1.46  2002/10/11 20:15:18  sacha
 * preview now exclude some state keys to force retrievem them from original mapfile.
 *
 * Revision 1.45  2002/09/05 14:16:39  bronsema
 * temporarily removed the delayed load so that errors in loading the map will
 * be shown
 *
 * Revision 1.44  2002/08/08 17:16:36  bronsema
 * Added the generic drawMap.php
 *
 * Revision 1.43  2002/08/01 11:24:56  bronsema
 * finished addition of query tool
 *
 * Revision 1.42  2002/07/07 17:37:14  pspencer
 * fixed visibility of new layer problem
 *
 * Revision 1.41  2002/07/05 17:43:24  pspencer
 * fixed map redraw in HTML mode so no navigation happens
 *
 * Revision 1.40  2002/07/04 15:36:35  pspencer
 * added warning if tmp files can't be deleted
 *
 * Revision 1.39  2002/07/04 15:17:25  bronsema
 * updated to resolve some session vs non-session vars
 *
 * Revision 1.38  2002/07/03 01:24:29  bronsema
 * Formatted the "no keymap specified" message.
 *
 * Revision 1.37  2002/07/02 19:49:15  pspencer
 * miscellaneous fixes for session support
 *
 * Revision 1.36  2002/07/02 15:54:53  pspencer
 * more changes related to session handling
 *
 * Revision 1.35  2002/06/28 16:57:46  pspencer
 * preliminary code for non-cookie session support
 *
 * Revision 1.34  2002/06/25 18:09:37  pspencer
 * fixed typo that was causing php parse error
 *
 * Revision 1.33  2002/06/25 17:07:39  pspencer
 * fixed bug in switching to ROSA from HTML that re-applied the last navigation
 * command
 *
 * Revision 1.32  2002/06/13 14:12:25  sacha
 * Change copywrite msg
 *
 * Revision 1.31  2002/06/07 14:02:21  pspencer
 * changed name of session variable for navigation mode to match mapbrowser
 *
 * Revision 1.30  2002/05/28 20:34:27  pspencer
 * fixed spelling error
 *
 * Revision 1.29  2002/05/28 14:29:54  pspencer
 * made HTML zoomin/zoomout images clickable
 *
 * Revision 1.28  2002/05/28 11:58:00  sacha
 * Fixed bug 1034
 *
 * Revision 1.27  2002/05/24 12:58:46  pspencer
 * moved all preview code into preview.phtml
 *
 * Revision 1.26  2002/05/23 19:22:54  pspencer
 * new handling of session vars and new name for gContext
 *
 * Revision 1.25  2002/05/17 02:13:57  pspencer
 * added support for HTML mode in preview
 *
 * Revision 1.24  2002/05/14 20:58:24  pspencer
 * added keymap navigation
 *
 * Revision 1.23  2002/05/14 15:04:21  pspencer
 * handle layers with spaces in the name
 *
 * Revision 1.22  2002/05/10 19:08:27  pspencer
 * modified calculation of location of legend_template.
 *
 * Revision 1.21  2002/05/10 18:27:02  pspencer
 * added user input for scale and extents in preview
 *
 * Revision 1.20  2002/05/10 15:11:18  pspencer
 * fixed include paths to use COMMON
 *
 * Revision 1.19  2002/05/08 20:25:56  sacha
 * Remove check of ImageType since it is ignored by mapscript.
 *
 * Revision 1.18  2002/05/08 14:00:45  sacha
 * Only load php mapscript module if not already loaded.
 *
 * Revision 1.17  2002/05/07 15:54:45  sacha
 * Check if REFERENCE MAP is set before drawing it.
 *
 * Revision 1.16  2002/05/07 14:47:16  sacha
 * Set image path and tmp image url to the same in .xml config file if not set
 * in mapfile.
 *
 * Revision 1.15  2002/05/03 13:47:04  pspencer
 * fixed navigation
 *
 * Revision 1.14  2002/05/01 18:18:11  sacha
 * Remove temp map file when finished.
 *
 * Revision 1.13  2002/04/25 00:05:02  pspencer
 * added calculation for current extents and map scale for preview display
 *
 * Revision 1.12  2002/04/19 19:22:19  pspencer
 * changed preview size to use map file defined sizes
 *
 * Revision 1.11  2002/04/17 19:07:44  pspencer
 * removed debug
 *
 * Revision 1.10  2002/04/17 19:06:39  pspencer
 * removed debug
 *
 * Revision 1.9  2002/04/17 19:05:13  pspencer
 * changed img format for applet to GIF
 *
 * Revision 1.8  2002/04/17 15:31:06  pspencer
 * added a full_extents button and code
 *
 * Revision 1.7  2002/04/16 20:33:31  pspencer
 * modified layout
 *
 * Revision 1.6  2002/04/12 16:03:21  sacha
 * Add two variables for tmp image path and tmp image url.
 *
 * Revision 1.5  2002/04/08 00:29:11  pspencer
 * fixed misspelled param name
 *
 * Revision 1.4  2002/04/02 15:40:25  pspencer
 * fixed layer visibility issues
 *
 * Revision 1.3  2002/04/02 01:46:50  pspencer
 * some more bug fixes
 *
 * Revision 1.2  2002/04/01 21:01:11  pspencer
 * many bug fixes
 *
 * Revision 1.1  2002/04/01 15:04:33  pspencer
 * new file
 *
 *
 *****************************************************************************/

// include the globprefs file
include_once( "globprefs.php" );

//sanity check
if ($_SESSION["gszMap"] == "")
{
    echo "No map file or map file empty!";
    exit;
}

//include mapscript wrapper classes
include_once( COMMON."wrapper/utilities.php" );
include_once( COMMON."wrapper/map_session.php" );
include_once( COMMON."wrapper/map_navigator.php" );

$szRosaFile = $_SESSION["gMapEditContext"]->getContextValue("rosa_file");
$nZoomFactor = $_SESSION["gMapEditContext"]->getContextValue("zoom_factor");
$nMinZoomRect = $_SESSION["gMapEditContext"]->getContextValue("min_zoom_rect");
//load the php module
/* -------------------------------------------------------------------- */
/*      Load required modules                                           */
/* -------------------------------------------------------------------- */
if (PHP_OS == "WINNT" || PHP_OS == "WIN32")
{
    if (!extension_loaded("MapScript"))
    {
        /**
         * Check if a module name is specified.
         * If so, use it. If not takes the default
         * one.
         */
        if (!isset($_SESSION["gszPHPMapScriptModName"]))
            $_SESSION["gszPHPMapScriptModName"] = "php_mapscript.dll";

        dl($_SESSION["gszPHPMapScriptModName"]);
    }
}
else
{
    if (!extension_loaded("MapScript"))
    {
        /**
         * Check if a module name is specified.
         * If so, use it. If not takes the default
         * one.
         */
        if (!isset($_SESSION["gszPHPMapScriptModName"]))
            $_SESSION["gszPHPMapScriptModName"] = "php_mapscript.so";

        dl($_SESSION["gszPHPMapScriptModName"]);
    }
}

// save a temp copy of the map file
$szTempDir = $_SESSION["gszTmpMapPath"];
$szTempMap = $szTempDir."/".session_id().".map";
$hMap = fopen( $szTempMap, "w+" );
fwrite( $hMap, $_SESSION["gszMap"] );
fclose($hMap);


// create a new map session object
$oMapSession = new MapSession_R;
$oMapSession->setTempDir( $szTempDir );

// set the maximum extents to limit navigation
//if ( count($afMapExtents) == 4 )
//    $oMapSession->setMaxExtents( $afMapExtents[0], $afMapExtents[1],
//                             $afMapExtents[2], $afMapExtents[3] );

// create a new map navigator object
$oMapNavigator = new MapNavigator( $oMapSession );
/* ============================================================================
 * Build an array with the HTTP GET or POST parameters.
 * ========================================================================= */
$http_form_vars = (count($_POST) > 0 ) ? $_POST : ( (count($_GET)) > 0 ? $_GET : array() );


if (isset($http_form_vars["ZOOM_FACTOR"]))
{
    $nZoomFactor = $http_form_vars["ZOOM_FACTOR"];
}
/* ============================================================================
 * Re-instate the previous map state
 * ========================================================================= */
// determine previous state key
if ( isset( $http_form_vars["PrevStateKey"] ) )
    $szPrevStateKey = urldecode($http_form_vars["PrevStateKey"]);
else
    $szPrevStateKey = "";

//load the map file
$oMapSession->restoreState( "", $szTempMap, dirname($_SESSION["gszMapName"]));
$oMapExtents = $oMapSession->oMap->extent;
$nMapWidth = $oMapSession->oMap->width;
$nMapHeight = $oMapSession->oMap->height;

$nReferenceWidth = $oMapSession->oMap->reference->width;
$nReferenceHeight = $oMapSession->oMap->reference->height;

// restore the previous state but don't set the layer status'
$oMapSession->restoreState( $szPrevStateKey, $szTempMap, dirname($_SESSION["gszMapName"]), false );
// Check if image path and tmp urd directory is set.
// If not use the one specified in .xml config file.
if ($oMapSession->oMap->web->imagepath == "" && $_SESSION["gszTmpImgPath"] != "")
    $oMapSession->oMap->web->set("imagepath", $_SESSION["gszTmpImgPath"]);

if ($oMapSession->oMap->web->imageurl == "" && $_SESSION["gszTmpWebImgPath"] != "")
    $oMapSession->oMap->web->set("imageurl", $_SESSION["gszTmpWebImgPath"]);


if ( strlen(trim($oMapSession->oMap->legend->template)) > 0 &&
     file_exists(trim($oMapSession->oMap->legend->template)))
     $szLegendTemplateFile = trim($oMapSession->oMap->legend->template);
else
  $szLegendTemplateFile = getcwd()."/legend_template.html";

/** MOVED TO THE BOTTOM OF PHTML PAGE TO ALLOW drawMap.php access
//remove the temp map obj
$nStatus = @unlink($szTempMap);
if ($nStatus === false)
{
    $_SESSION["gErrorManager"]->error( ERR_NOTICE,
                "unable to delete temp file $szTempFile" );
}
*/

// set the height and width of the map
//$oMapSession->oMap->set( "width", $nMapWidth );
//$oMapSession->oMap->set( "height", $nMapHeight );

// log event
//$oMapSession->log( LOG_QUIET, "Set the map width and height to ".
//                               "$nMapWidth,$nMapHeight" );

$szLayerList = "";
$szSep = "";
for($i=0;$i<$oMapSession->oMap->numlayers;$i++)
{
    $oLayer = $oMapSession->oMap->getlayer($i);
    $szLayerList .= $szSep.$oLayer->name;
    $szSep = "|";
}


/* ============================================================================
 * Check for navigation mode
 * ========================================================================= */
if (isset($http_form_vars['NavigationMode']) &&
    $http_form_vars['NavigationMode'] != $_SESSION["gszNavigationMode"])
{
        $_SESSION["gszNavigationMode"] = $http_form_vars['NavigationMode'];
        $http_form_vars['ROSA_CMD'] = "";
        unset($http_form_vars["MAINMAP_x"]);
        unset($http_form_vars["MAINMAP_y"]);
}

/* ============================================================================
 * Check if the mainmap was clicked and update the co-ordinates
 * ========================================================================= */
if ( isset( $http_form_vars[ "MAINMAP_x" ] ) &&
     isset( $http_form_vars[ "MAINMAP_y" ] ) )
{
    // set the values into the nav hidden variable
    $http_form_vars["INPUT_COORD"] = $http_form_vars[ "MAINMAP_x" ].",".
                                     $http_form_vars[ "MAINMAP_y" ];
}

/* ============================================================================
 * Process navigation commands.
 * ========================================================================= */

if(isset($http_form_vars["APPLY_SCALE_x"]))
{
    //zoom to scale
    $oMapNavigator->zoomScale($http_form_vars["MAP_SCALE"]);
}
else if(isset($http_form_vars["APPLY_EXTENTS_x"]))
{
    //zoom to extents
    $oMapSession->oMap->setextent( $http_form_vars["MAP_MINX"],
                                   $http_form_vars["MAP_MINY"],
                                   $http_form_vars["MAP_MAXX"],
                                   $http_form_vars["MAP_MAXY"] );
}
elseif (isset($http_form_vars['KEYMAP_NAV_x']))
{
    keymapNavigate( $oMapSession, $oMapNavigator,
                    $http_form_vars['KEYMAP_NAV_x'],
                    $http_form_vars['KEYMAP_NAV_y'],
                    $oMapExtents, $nReferenceWidth, $nReferenceHeight );
}
elseif(isset($http_form_vars['FULLEXTENTS_x']))
{
    $oMap = $oMapSession->oMap;
    $oMap->setExtent( $oMapExtents->minx, $oMapExtents->miny, $oMapExtents->maxx, $oMapExtents->maxy );

}
elseif(isset($http_form_vars['HTML_ZOOMIN_x']))
{
    $oMapNavigator->zoomIn( 2 );
}
elseif(isset($http_form_vars['HTML_ZOOMOUT_x']))
{
    $oMapNavigator->zoomOut( 2 );
}
elseif(isset( $http_form_vars["redraw_x"] ))
{
    //pass to ignore current command since we are just redrawing
}
/*elseif (!checkNavigation($oMapNavigator,$http_form_vars["ROSA_CMD"],
                                $http_form_vars["INPUT_COORD"], $oMapQuery))
{
    // give error if failed
    $oMapNavigator->error(ERROR_NOTICE,"checkNavigation Failed");
}*/

// process navigation requests
if ( ( ( $_SESSION["gszNavigationMode"] != "ROSA" &&
       isset( $http_form_vars[ "MAINMAP_x" ] ) ) ||
       ( $_SESSION["gszNavigationMode"] == "ROSA" ) &&
        isset( $http_form_vars["INPUT_COORD"] ) &&
        $http_form_vars["INPUT_COORD"] != "" ))

{
    // check the naviagtion command and process
    $xNavigate = checkNavigation($oMapNavigator,$http_form_vars["ROSA_CMD"],
                                $http_form_vars["INPUT_COORD"], NULL);//$oMapQuery);

    // set the query string variable
    if ( strlen( $xNavigate ) > 1 )
        $szQueryString = $xNavigate;
    else
        $szQueryString = "";

    // check for success
    if (!$xNavigate )
    {
        // give error if failed
        $oMapNavigator->error(ERROR_NOTICE,"checkNavigation Failed");
    }
}

/* ============================================================================
 * Process the layer status.
 * ========================================================================= */
updateLayerStatus( $oMapSession->oMap, $http_form_vars );

/* ============================================================================
 * Process the legend template
 * ========================================================================= */
$oLegend = $oMapSession->oMap->legend->set("template", $szLegendTemplateFile);

$szLegendTemplate = $oMapSession->processLegendTemplate();


/* ============================================================================
 * Save the current state.
 * ========================================================================= */
$szCurrentState = $oMapSession->saveState(array("MAPSIZE", "SRS"));

/* ============================================================================
 * Render the map.
 * ========================================================================= */
$szURL = "http://".$HTTP_SERVER_VARS['HTTP_HOST'];

$imgMainMap = $oMapSession->oMap->draw();
$nScale = $oMapSession->oMap->scale;
$oCurExtent = $oMapSession->oMap->extent;

//if (!session_is_registered("gszImageType"))
    $urlMainMap = $imgMainMap->saveWebImage();
//else
//    eval("\$urlMainMap = \$imgMainMap->saveWebImage(MS_".strtoupper($_SESSION['gszImageType']).", 0, 0, -1);");
$urlMainMap = $szURL.$urlMainMap;

$imgScalebar = $oMapSession->oMap->drawScalebar();

$urlScalebar = $imgScalebar->saveWebImage();

$urlScalebar = $szURL.$urlScalebar;

if ($oMapSession->oMap->reference->image != "")
{
    $imgReference = $oMapSession->oMap->drawReferenceMap();

    $nReferenceWidth = $imgReference->width;
    $nReferenceHeight = $imgReference->height;

    $urlReference = $imgReference->saveWebImage();
    $urlReference = $szURL.$urlReference;
}

// check if reference values are valid
if ( !is_numeric( $nReferenceWidth ) || $nReferenceWidth < 1 )
    $nReferenceWidth = 100;
if ( !is_numeric( $nReferenceHeight ) || $nReferenceHeight < 1 )
    $nReferenceHeight = 75;

/**
 * This function determines which navigation button is to be selected
 *
 * @param $szRosaCmd String - value of 'ROSA_CMD' form variable
 * @return String - value to be selected
 **/
function checkSelected($szRosaCmd)
{
    switch ($szRosaCmd)
    {
    case "ZOOM_IN":
        return "zoomin";
        break;
    case "ZOOM_OUT":
        return "zoomin";
        break;
    case "RECENTRE":
        return "recentre";
        break;
    // query
    case "QUERY":
        return "pquery";
        break;
     default:
        return "zoomin";
        break;
    }
}

/**
 * This function processes the navigation commands
 *
 * @param $oMapNavigator
 * @param $szRosaCmd
 * @param $szInputCoords
 * @return Boolean - true if navigation selected
 **/
function checkNavigation($oMapNavigator,$szRosaCmd, $szInputCoords, $oMapQuery)
{
    global $nMinZoomRect, $nZoomFactor, $oMapExtents;

    //only process if coordinates are given
    if (isset($szInputCoords))
    {
        //process coordinates
        $aszMinMax = explode(";", $szInputCoords);
        if(sizeof($aszMinMax) < 2)
        {   //bad coords
            return false;
        }
        $adMin = explode(",", $aszMinMax[0]);
        $adMax = explode(",", $aszMinMax[1]);
    }
    //process indiv commands
    switch ($szRosaCmd)
    {
    case "FULLEXTENTS":
        $oMap = $oMapNavigator->oSession->oMap;
        $oMap->setExtent( $oMapExtents->minx, $oMapExtents->miny, $oMapExtents->maxx, $oMapExtents->maxy );
        return true;
        break;
    case "ZOOM_IN":
        //check if for point click
        $nDeltaX = $adMax[0] - $adMin[0];
        $nDeltaY = $adMax[1] - $adMin[1];

        if ($nDeltaX < $nMinZoomRect && $DeltaY < $nMinZoomRect)
            $oMapNavigator->zoomPoint($nZoomFactor, $adMin[0], $adMin[1]);
        else
            $oMapNavigator->zoomRectangle($adMin[0],$adMin[1],$adMax[0],
                                                                $adMax[1]);
        return true;
        break;
    case "ZOOM_OUT":
        $oMapNavigator->zoomOut($nZoomFactor);
        return true;
        break;
    case "RECENTRE":
        $oMapNavigator->recentre($adMin[0],$adMin[1]);
        return true;
        break;

    // query
    case "QUERY":

        // return results
        return "QUERY";

        // exit switch
        break;
    default:
        return false;
    }
}


/**
 * This function updates the layer status in the map file according to the
 * user's selection in the URL.
 *
 * @param oMap object - The map object to update
 * @param szURL string - The URL to parse.
 **/
function updateLayerStatus( $oMap, $szURL )
{

    // initialize the varsiable
    $aszLayers = array();

    if (isset($szURL['LayerList']))
        $aszLayerList = explode( "|", urldecode($szURL['LayerList']));
    else
        $aszLayerList = array();

    //get the layer status
    if (isset($szURL['layer_status']))
    {
        $aszLayers = $szURL['layer_status'];
    }
    else
    {
        $aszLayers = array();
    }

    if (count($szURL)!=0)
    {
        // loop through each layer and set it's status
        for ( $i=0; $i<$oMap->numlayers; $i++ )
        {
            // create a layer object
            $oLayer = $oMap->getlayer( $i );
            // check if the name is in the URL
            if ( in_array($oLayer->name, $aszLayers) )
            {
                // set layer to ON
                $oLayer->set("status",MS_ON);
            }
            elseif( in_array($oLayer->name, $aszLayerList) )
            {
                // set layer to OFF
                $oLayer->set("status",MS_OFF);
            }
        }
    }
}

/**
 * This function takes the keymap click co-ordinates and recenters the map
 * accordingly.
 *
 * @param $oSession object - The map session object.
 * @param $oNavigate object - The map navigation object.
 * @param $nX integer - The keymap X click position in pixels.
 * @param $nY integer - The keymap Y click position in pixels.
 * @param $oExtents - the extents that the keymap represent
 **/
function keymapNavigate ( $oSession, $oNavigate, $nX, $nY, $oExtents,
                          $nKeymapWidth, $nKeymapHeight )
{
    // log entry
    $oSession->logFuncStart( LOG_VERBOSE, "keymapNavigate() starting" );

    // get local map object
    $oMap = $oSession->getMapObj();

    // get x click position
    $nMapXClick = keymapToMap( $oMap->width, $oMap->extent->minx,
                        $oMap->extent->maxx, $oExtents->minx, $oExtents->maxx, $nX,
                                            $nKeymapWidth, $oSession, true);
    // get y click position
    $nMapYClick = keymapToMap( $oMap->height, $oMap->extent->miny,
                        $oMap->extent->maxy, $oExtents->miny, $oExtents->maxy, $nY,
                                            $nKeymapHeight, $oSession, false);
    // recenter to the click position
    $oNavigate->recentre( $nMapXClick, $nMapYClick );

    // log the x and y parameters
    $oSession->log( LOG_QUIET, "X & Y Parameters are: $nMapXClick,
                                                                $nMapYClick");
    // log exit
    $oSession->logFuncEnd( LOG_ALL, "keymapNavigate() done" );
}

/**
 * This function calculates the corresponding map click given a keymap click.
 *
 * @param $nMapPix integer - The map width or height in pixels.
 * @param $nMapGeo integer - The change in current map extents
 *                              (i.e. maxX - minX).
 * @param $nMapGeoMax integer - The change in max map extents.
 * @param $nKeyPixClick integer - The keymap click position in pixels.
 * @param $nKeyPix integer - The width or height of the keymap in pixels.
 * @return integer - The map click position in pixels.
 **/
function keymapToMap ( $nMapPix, $nMapMinGeo, $nMapMaxGeo, $nMapMinGeoMax,
                        $nMapMaxGeoMax, $nKeyPixClick, $nKeyPix, $oSession,
                        $bX )
{
    // log entry
    $oSession->logFuncStart( LOG_VERBOSE, "keymapToMap() starting" );

    // calculate the difference in extents
    $nMapGeo = $nMapMaxGeo - $nMapMinGeo;
    $nMapGeoMax = $nMapMaxGeoMax - $nMapMinGeoMax;

    // calculate the pixel value of the maximum map extents (i.e. width or
    // height of map at maximum extents)
    $nMapPixMax = ( $nMapPix * $nMapGeoMax ) / $nMapGeo;

    // calculate the map click distance
    $dClickDistance =  ( $nKeyPixClick * $nMapPixMax ) / $nKeyPix;

    // calculate the value to translate the origin
    if ($bX)
        $dDeltaOrigin = (($nMapMinGeo - $nMapMinGeoMax) * $nMapPix) / $nMapGeo;
    else
        $dDeltaOrigin = (($nMapMaxGeoMax - $nMapMaxGeo) * $nMapPix) / $nMapGeo;

    // calculate the return value
    $dReturnValue = $dClickDistance - $dDeltaOrigin;

    // log exit
    $oSession->logFuncEnd( LOG_ALL, "keymapToMap() done" );

    // return the results
    return $dReturnValue;

// end keymapToMap function
}
?>













